/*      
        alph: Piper program implementing and analysing traditional and
	    historic c(ai)phers designed to be used in conjuction with other
    	programs.

        Copyright (C) 2004  Corcalciuc V. Horia

        This library is free software; you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public
        License as published by the Free Software Foundation; either
        version 2.1 of the License, or (at your option) any later version.

        This library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with this library; if not, write to the Free Software
        Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

        Dedicated to Rudolf.
*/


/* System. */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <string.h>
#include <popt.h>

/* Our own. */
#include "alphCRYPT.h"
#include "alphan.h"
#include "config.h"

int
main (const int argc, const char **argv)
{

  poptContext optCon;
  char *optBuf;
  int result;
  int itra = 0;
  int status, pid;
  struct akeys {
    char *k1;
    char *k2;
    char *k3;
    char *k4;
  } key;
  char *indata;
  FILE *fp;

  typedef enum
  {
    false = 0,
    true
  } flip;
  flip check = false, dsmash = false;

  const struct poptOption optionT[] = {
    {"\b\b  \nCyphers:", 0, 0, NULL, 0, NULL},
    {"caesar", 0, 0, NULL, '0', "CAESAR", NULL},
    {"atbash", 0, 0, NULL, '1', "ATBASH", NULL},
    {"rot13", 0, 0, NULL, '2', "ROT-13", NULL},
    {"vigenere", 0, POPT_ARG_STRING, &key.k1, '3', "VIGENERE", "KEY"},
    {"vernam", 0, POPT_ARG_STRING, &key.k1, '4', "VERNAM", "FILE"},
    {"playfair", 0, POPT_ARG_STRING, &key.k1, '5', "PLAYFAIR", "KEY"},
    {"enigma", 0, POPT_ARG_STRING, &key.k1 , '6', "ENIGMA", "ROTOR,REFLECTOR,RING"},
    {"\b\b  \nOptions:", 0, 0, NULL, 0, NULL},
    {NULL, 's', 0, NULL, 's', "attempt smash", NULL},
    {NULL, 'd', 0, NULL, 'd', "decrypt text", NULL},
    {NULL, 'e', 0, NULL, 'e', "encrypt text", NULL},
    {"\b\b  \nHelp:", 0, 0, NULL, 0, NULL},
    {"help", 'h', 0, NULL, 'h', "show help message", NULL},
    {"man", 0, 0, NULL, 'm', "display manual page", NULL},
    {NULL, 0, 0, NULL, 0}
  };

  optBuf = calloc (argc, sizeof (char));
  indata = calloc (BUFSIZ, sizeof (char));
  
  key.k1 = calloc (BUFSIZ, sizeof (char));
  key.k2 = calloc (BUFSIZ, sizeof (char));
  key.k3 = calloc (BUFSIZ, sizeof (char));
  key.k4 = calloc (BUFSIZ, sizeof (char));

  setvbuf (stdout, (char *) NULL, _IONBF, 0);
  setvbuf (stdin, (char *) NULL, _IONBF, 0);

  optCon = poptGetContext (NULL, argc, argv, optionT, 0);

  while ((result = poptGetNextOpt (optCon)) >= 0)
    {
      switch (result)
	    {
          case '0':
            optBuf[itra++]= '0';
            break;
          case '1':
            optBuf[itra++]= '1';
            break;
          case '2':
            optBuf[itra++]= '2';
            break;
          case '3':
            optBuf[itra++]= '3';
            break;
          case '4':
            optBuf[itra++]= '4';
            break;
          case '5':
            optBuf[itra++]= '5';
            break;
          case '6':
            optBuf[itra++]= '6';
            break;
            
          case 's':
            optBuf[itra++]= 's';
            break;
          case 'd':
            optBuf[itra++]= 'd';
            break;
          case 'e':
            optBuf[itra++]= 'e';
            break;

          case 'h':
            optBuf[itra++]= 'h';
            break;
          case 'm':
            optBuf[itra++]= 'm';
            break;
	    }
    }

  if (result < -1)
    {
      fprintf (stderr, "%s: %s\n",
	       poptBadOption (optCon, POPT_BADOPTION_NOALIAS),
	       poptStrerror (result));
      exit (1);
    }
  else if (argc < 2)
    {
      fprintf (stderr, "%s: You must select an action.\n", PACKAGE_NAME);
      exit (1);
    }

  if (strchr (optBuf, 'h') != NULL)
    {
      poptPrintHelp (optCon, stderr, 0);
      exit (0);
    }

  if (strchr (optBuf, 'm') != NULL)
    {
      if ((pid = vfork ()) == 0)
	    execlp ("man", "-a", PACKAGE_NAME, NULL);
      else if (pid < 0)
	    fprintf (stdout, "%s", strerror (errno));
      while (wait (&status) < 0);
      exit (0);
    }

  if (strchr (optBuf, 'd') != NULL)
    {
      al_dec = true;
    }
  else if (strchr (optBuf, 'e') != NULL)
    {
      al_dec = false;
    }
  else if (strchr (optBuf, 's') != NULL)
    {
      dsmash = true;
    }

  for (itra = 0; itra <= BUFSIZ; itra++)
  if (!feof (stdin))
    indata[itra] = fgetc (stdin);
  else
    {
      indata[itra - 1] = '\0';
      break;
    }
  

  if (strchr (optBuf, '0') != NULL)
    {
    	result = caesar (indata);
    }
  if (strchr (optBuf, '1') != NULL)
    {
    	result = atbash (indata);
    }
  if (strchr (optBuf, '2') != NULL)
    {
    	result = rot13 (indata);
    }
  if (strchr (optBuf, '3') != NULL)
    {
    	result = vigenere (indata, key.k1);
    }
  if (strchr (optBuf, '4') != NULL)
    {
      if ((fp = fopen (key.k1, "r")) == NULL)
    	fprintf (stdout, "%s : %s", PACKAGE_NAME, KEYB), exit (1);

      itra = 0;
      while ((result = fgetc (fp)) != EOF && itra < strlen (indata))
    	if (isalpha (result))
	      key.k1[itra++] = result;

      fclose (fp);

    	result = vernam (indata, key.k1);
    }
  if (strchr (optBuf, '5') != NULL)
    {
    	result = playfair (indata, key.k1);
    }
  if (strchr (optBuf, '6') != NULL)
    {
      if (strchr(key.k1, ',')==NULL)
        exit(1);
      else 
        {
          result=0;
          itra=0;
          while (itra<strlen(key.k1) && key.k1[itra]!=',')
            key.k2[result++]=key.k1[itra++];
          result=0;
          itra++;
          while (itra<strlen(key.k1) && key.k1[itra]!=',')
            key.k3[result++]=key.k1[itra++];
          result=0;
          itra++;
          while (itra<strlen(key.k1) && key.k1[itra]!=',')
            key.k4[result++]=(int) key.k1[itra++];
          result = enigma(indata, key.k2, key.k3, key.k4);
        }
    }

  if (dsmash)
    {
      fprintf (stdout, "%s", smash(indata));
    }
  else
    !result && fprintf (stdout, "%s", indata);

  poptFreeContext (optCon);
  exit (0);
}
